
/* Event definitions from Xwindows */
#define  XKeyPress	    2
#define  XKeyRelease	    3
#define  XButtonPress	    4
#define  XButtonRelease	    5
#define  XMotionNotify 	    6
#define  XEnterNotify	    7
#define  XLeaveNotify	    8
#define  XFocusIn	    9
#define  XFocusOut	   10
#define  XKeymapNotify	   11
#define  XExpose	   12
#define  XGraphicsExpose   13
#define  XNoExpose	   14
#define  XCreateNotify	   16
#define  XDestroyNotify	   17
#define  XUnmapNotify	   18
#define  XMapNotify	   19
#define  XMapRequest	   20
#define  XReparentNotify   21
#define  XConfigureNotify  22
#define  XConfigureRequest 23
#define  XGravityNotify    24
#define  XResizeRequest	   25
#define  XCirculateNotify  26
#define  XCirculateRequest 27
#define  XPropertyNotify   28
#define  XSelectionClear   29
#define  XSelectionRequest 30
#define  XSelectionNotify  31
#define  XColormapNotify   32
#define  XClientMessage	   33
#define  XMappingNotify	   34
#define  XLASTEvent	   35

/* Graphics functions (src to dest) from Xwindows */
#define  GXclear	  0
#define  GXand		  1
#define  GXandReverse	  2
#define  GXcopy		  3
#define  GXandInverted	  4
#define  GXnoop		  5
#define  GXxor		  6
#define  GXor		  7
#define  GXnor		  8
#define  GXequiv	  9
#define  GXinvert	 10
#define  GXorReverse	 11
#define  GXcopyInverted	 12
#define  GXorInverted	 13
#define  GXnand		 14
#define  GXset		 15

/* Synchronization functions from Xwindows */
#define  XFDIOff        0 
#define  XFDIOn         1 
#define  XFDINotThere   2 


