      PARAMETER (NMAX=2001,NRMAX=3)
      COMPLEX A,B,C,R, F0,F1,F2,F3
      COMPLEX FNORM,IRE, ALPHA,DALPHA, OMEGA,DOMEGA, ALINIT,OMINIT
      COMPLEX FAC, FACSQ, FAC_AL, FAC_OM, FAC_RE
C
      COMMON/OS_CPX/
     &    A(4,4,NMAX),B(4,4,NMAX),C(4,4,NMAX),R(4,NRMAX,NMAX),
     &    F0(NMAX),F1(NMAX),F2(NMAX),F3(NMAX),FNORM,
     &    IRE,ALPHA,DALPHA, OMEGA,DOMEGA, ALINIT,OMINIT,
     &    FAC, FACSQ, FAC_AL, FAC_OM, FAC_RE
      COMMON/OS_REL/
     &    Y(NMAX),U(NMAX),UD(NMAX),
     &    RLX,DFMAX,DFRMS, RESMAX,RESRMS
      COMMON/OS_INT/
     &    LST, LRE, N, NRHS, ITER, ITMAX, IBC,ISOL
