C
      PARAMETER (NAX = 80, NX=132, NXB=250, NFX=500)
      IMPLICIT REAL(M)
      CHARACTER*8 CODE
      CHARACTER*32 NAME
      LOGICAL LREF, LFORCE, LMACH, LISES, LCLFIX, LALFIX, LPLOT
      INTEGER MATYP,RETYP
C
      COMMON/COMI/
     &  IDEV, IDEVRP, IPSLU, N, NAPLT, NA, MATYP, RETYP,
     &  LREF, LFORCE, LMACH, LISES, LCLFIX, LALFIX, LPLOT,
     &  IAPLT(NAX), NF,
     &  II(2,NAX), IIB,
     &  ILE(2,NAX),ITE(2,NAX),
     &  ITRAN(2,NAX),
     &  CODE, 
     &  NAME

C
      COMMON/COMR/
     &  SIZE, SCRNFR,
     &  CLMIN, CLMAX, CDMIN, CDMAX, CMMIN, CMMAX,
     &  MACH, REYN, ACRIT, GAM,GM1,CPSTAR(NAX),
     &  MA(NAX),ALFA(NAX),CL(NAX),CD(NAX),CDI(NAX),CM(NAX),
     &  APLT(NAX),
     &  XTR(2,NAX),
     &  XF(NFX),MF(NFX),
     &  XB(NXB), YB(NXB),
     &  X(NX,2,NAX),CP(NX,2,NAX),
     &  THET(NX,2,NAX),DSTR(NX,2,NAX),CF(NX,2,NAX),CTAU(NX,2,NAX),
     &  VERSION

C
